@@ -19,7 +19,7 @@
 * License along with this library; if not, write to the Free Software           *
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA.    *
 *********************************************************************************
-* $Id: FXFont.cpp,v 1.184 2006/01/22 17:58:26 fox Exp $                         *
+* $Id: FXFont.cpp,v 1.184.2.3 2007/04/29 14:31:44 fox Exp $                         *
 ********************************************************************************/
 #include "xincs.h"
 #include "fxver.h"
@@ -696,7 +696,7 @@ void* FXFont::match(const FXString& wantfamily,const FXString& wantforge,FXuint
 
   // Open font
   font=XftFontOpenPattern(DISPLAY(getApp()),p);
-  xid=(unsigned long)p;
+  xid=(FXID)font;
 
   // Destroy pattern
   FcPatternDestroy(pattern);
@@ -910,7 +910,7 @@ void* FXFont::match(const FXString& wantfamily,const FXString& wantforge,FXuint
     // Match them
     for(f=0; f<nfontnames; f++){
 
-      //FXTRACE((1,"font=%s\n",fontnames[f]));
+      //FXTRACE((100,"font=%s\n",fontnames[f]));
 
       // Break apart into fields
       strncpy(candidate,fontnames[f],sizeof(candidate));
@@ -1128,22 +1128,19 @@ void* FXFont::match(const FXString& wantfamily,const FXString& wantforge,FXuint
         font=XLoadQueryFont(DISPLAY(getApp()),xlfd);
         }
 
-      // Remember font id
-      if(font){ xid=((XFontStruct*)font)->fid; }
-
 /*
       if(font){
         for(b=0; b<((XFontStruct*)font)->n_properties; b++){
           if(((XFontStruct*)font)->properties[b].name==XA_FONT){
             char *fn=XGetAtomName(DISPLAY(getApp()),((XFontStruct*)font)->properties[b].card32);
             strncpy(candidate,fn,sizeof(candidate));
-            FXTRACE((1,"FONT = %s\n",candidate));
+            FXTRACE((100,"FONT = %s\n",candidate));
             xlfdSplit(field,candidate);
             XFree(fn);
-            FXTRACE((1,"mat  = %s\n",field[6]));
+            FXTRACE((100,"mat  = %s\n",field[6]));
             }
           else{
-            FXTRACE((1,"%d (%s) = %d\n",((XFontStruct*)font)->properties[b].name,XGetAtomName(DISPLAY(getApp()),((XFontStruct*)font)->properties[b].name),((XFontStruct*)font)->properties[b].card32));
+            FXTRACE((100,"%d (%s) = %d\n",((XFontStruct*)font)->properties[b].name,XGetAtomName(DISPLAY(getApp()),((XFontStruct*)font)->properties[b].name),((XFontStruct*)font)->properties[b].card32));
             }
           }
         }
@@ -1410,6 +1407,9 @@ void FXFont::create(){
         font=XLoadQueryFont(DISPLAY(getApp()),actualName.text());
         }
 
+      // Remember font id
+      if(font){ xid=((XFontStruct*)font)->fid; }
+
       // Uh-oh, we failed
       if(!xid){ throw FXFontException("unable to create font"); }
 
